// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __frame3d_h__
#define __frame3d_h__

#include "frame3dbase.h"
#include "colorscale.h"

class Frame3d : virtual public Frame3dBase {
 protected:
  FitsImage* fits;           // list of fits images

  int mosaicCount;             // count of fits images
  int sliceCount;             // count of fits images

  int cmapID;                // current colormap id
  float bias;                // current colormap bias
  float contrast;            // current colormap contrast

  int colorCount;            // number of dynamic colors
  ColorScale* colorScale;    // current color scale
  unsigned char* colorCells; // current color values
  unsigned short* indexCells; // current color indices

  FrScale frScale;           // scale parameters
  MosaicType mosaicType;
  CoordSystem mosaicSystem;

 private:
  void alignWCS();
  void alignWCS(FitsImage*);

  int doRender() {return currentFits ? 1 : 0;}

  void reset();

  void setCurrentFits(FitsImage* f) {currentFits = f;}
  void setKeyFits() {};

  void updateBin(const Matrix&);
  void updateMagnifierMatrices(Matrix& refToMagnifier);
  void updateMatrices();
  void unload();
  void unloadFits();

 protected:
  Vector3d imageSize();
  int isIIS() {return 0;}

  void unloadAllFits() {unloadFits();}
  void updateColorCells(unsigned short*, unsigned char*, int);

  int validColorScale() {return colorScale ? 1 : 0;}

 public:
  Frame3d(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame3d();

  void getColormapCmd();
  void getRGBChannelCmd();
  void getRGBSystemCmd();
  void getRGBViewCmd();
  void getTypeCmd();
  void getWCSAlignCmd();

  void sliceCmd(int);
};

#endif
